package controllers

import (
	"encoding/json"
	"hongze/hongze_edb_lib/logic"
	"hongze/hongze_edb_lib/models"
	"hongze/hongze_edb_lib/services"
	"hongze/hongze_edb_lib/utils"
	"strconv"
	"time"
)

// 指标
type EdbInfoController struct {
	BaseAuthController
}

// @Title 新增指标接口
// @Description  新增指标接口
// @Success 200 {object} models.AddEdbInfoParams
// @router /add [post]
func (this *EdbInfoController) Add() {
	br := new(models.BaseResponse).Init()
	var cacheKey string
	defer func() {
		utils.Rc.Delete(cacheKey)
		this.Data["json"] = br
		this.ServeJSON()
	}()

	var req models.AddEdbInfoParams
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.EdbCode == "" {
		br.Msg = "请输入指标编码！"
		br.ErrMsg = "请输入指标编码，指标编码为空"
		return
	}
	deleteCache := true
	cacheKey = "CACHE_EDB_INFO_ADD_" + req.EdbCode
	defer func() {
		if deleteCache {
			utils.Rc.Delete(cacheKey)
		}
	}()
	if !utils.Rc.SetNX(cacheKey, 1, 30*time.Second) {
		deleteCache = false
		br.Msg = "系统处理中，请稍后重试！"
		br.ErrMsg = "系统处理中，请稍后重试！"
		return
	}

	if req.ClassifyId <= 0 {
		br.Msg = "请输入指标分类！"
		br.ErrMsg = "请输入指标分类"
		return
	}
	if req.Source <= 0 {
		br.Msg = "请输入指标来源！"
		br.ErrMsg = "请输入指标来源"
		return
	}
	if req.EdbName == "" {
		br.Msg = "请输入指标名称！"
		br.ErrMsg = "请输入指标名称"
		return
	}
	if req.Frequency == "" {
		br.Msg = "请输入指标频度！"
		br.ErrMsg = "请输入指标频度"
		return
	}
	if req.Unit == "" {
		br.Msg = "请输入指标单位！"
		br.ErrMsg = "请输入指标单位"
		return
	}
	var isAdd bool
	item, err := models.GetEdbInfoByEdbCode(req.Source, req.EdbCode)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			isAdd = true
		} else {
			br.Msg = "判断指标是否存在失败！"
			br.ErrMsg = "判断指标是否存在失败"
			return
		}
	}
	if item != nil && item.EdbInfoId <= 0 {
		isAdd = true
	}
	var windUrl string
	if isAdd {
		if req.Source == utils.DATA_SOURCE_WIND {
			windUrl, err = services.GetWindUrl(req.EdbCode)
			if err != nil {
				br.Msg = "判断失败！"
				br.ErrMsg = "判断失败,Err:" + err.Error()
				return
			}
		}
	}
	admnId, _ := strconv.Atoi(utils.InitAdminId)
	edbInfo, err := models.EdbInfoAdd(&req, windUrl, admnId, utils.InitAdminName)
	if err != nil {
		br.Msg = "新增指标失败！"
		br.ErrMsg = "新增指标失败,Err:" + err.Error()
		return
	}
	// 更新ES
	go logic.UpdateEs(edbInfo.EdbInfoId)
	br.Ret = 200
	br.Msg = "新增成功"
	br.Data = edbInfo
}
